/* $Id: internal.h,v 1.8 2003/08/01 10:31:07 david Exp $
 *
 * midgard-lib: database access for Midgard clients
 *
 * Copyright (C) 1999 Jukka Zitting <jukka.zitting@iki.fi>
 * Copyright (C) 2000 The Midgard Project ry
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef MIDGARD_INTERNAL_H
#define MIDGARD_INTERNAL_H
#include "midgard/midgard.h"
#include "midgard/parser.h"

#include <mysql.h>
#if HAVE_ERRMSG_H
#include <errmsg.h>
#endif

extern void (*mgd_lib_log_debug)(int flags, const char *fmt, ...);

typedef struct _mgd_userinfo {
   int id, is_admin, *member_of;
#if HAVE_MIDGARD_SITEGROUPS
   int is_root, sitegroup;
#endif
} mgd_userinfo;

struct midgard {
   MYSQL *mysql;
   midgard_res *res;
   midgard_pool *pool, *tmp;
   mgd_parser* parser;

   mgd_userinfo user_at_pagestart, setuid_user;
   mgd_userinfo *current_user;

#if HAVE_MIDGARD_MULTILANG
   int lang;
  int p_lang;
  int a_lang;
#endif /* HAVE_MIDGARD_MULTILANG */
   char *username;
   char *blobdir;
#if HAVE_MIDGARD_MULTILANG
   char *cvs_script;
#endif /* HAVE_MIDGARD_MULTILANG */
};

struct midgard_res {
    midgard *mgd;
    midgard_res *next, *prev;
    midgard_pool *pool;

    MYSQL_RES *res;
    MYSQL_ROW row;
    MYSQL_FIELD *fields;
    int rown, rows, cols;
};

#if MIDGARD_GLIB_POOLS
struct midgard_pool {
   GSList *handle;
};
#else
struct midgard_pool {
    midgard_pool *next;
    void *data;
};
#endif

#define MGD_LIST_QUERY 0
#define MGD_GET_QUERY  1
#define MGD_STD_FIELDS "id,'0000-00-00 00:00:00','0000-00-00 00:00:00',0,0,0,"
#define MGD_CLEAR_STD_FIELDS(rec) do { \
  (rec)->id       = 0; \
  (rec)->created  = 0; \
  (rec)->creator  = 0; \
  (rec)->revised  = 0; \
  (rec)->revisor  = 0; \
  (rec)->revision = 0; \
} while (0)
#define MGD_GET_STD_FIELDS(rec) do { \
  (rec)->id       = mgd_sql2id((rec)->res, 0); \
  (rec)->created  = mgd_sql2datetime((rec)->res, 1); \
  (rec)->creator  = mgd_sql2id((rec)->res, 2); \
  (rec)->revised  = mgd_sql2datetime((rec)->res, 3); \
  (rec)->revisor  = mgd_sql2id((rec)->res, 4); \
  (rec)->revision = mgd_sql2int((rec)->res, 5); \
} while (0)
#define MGD_GET_ID_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2id((rec)->res, col+6))
#define MGD_GET_INT_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2int((rec)->res, col+6))
#define MGD_GET_STR_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2str((rec)->res, col+6))
#define MGD_GET_DATE_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2date((rec)->res, col+6))
#define MGD_GET_TIME_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2time((rec)->res, col+6))
#define MGD_GET_DATETIME_FIELD(rec, name, col) \
  ((rec)->name = mgd_sql2datetime((rec)->res, col+6))

#endif /* MIDGARD_INTERNAL_H */
